@@ -2,16 +2,20 @@ from . import acquisition
 from . import hdf_utils
 from . import pixel_utils
 from . import analysis
-from . import gkpfm
 from . import simulation
+from . import gkpfm
 from . import load
 
 from . import pixel
 from . import line
 
+from .__version__ import version as __version__
+
 __all__ = ['line', 'pixel']
 __all__ += acquisition.__all__
 __all__ += hdf_utils.__all__
 __all__ += pixel_utils.__all__
 __all__ += analysis.__all__
-__all__ += gkpfm.__all__
\ No newline at end of file
+__all__ += gkpfm.__all__
+__all__ += load.__all__
+__all__ += simulation.__all__
\ No newline at end of file
