@@ -5,7 +5,8 @@ Created on Tue Feb 11 18:07:06 2020
 @author: Raj
 """
 
-import pyUSID as usid
+from pyUSID.io.hdf_utils import write_main_dataset, find_dataset, create_results_group
+from pyUSID import Process
 import ffta
 from ffta.pixel import Pixel
 from ffta.pixel_utils import badpixels
@@ -13,13 +14,15 @@ import os
 import numpy as np
 from ffta.load import get_utils
 from sidpy.proc.comp_utils import parallel_compute
-from pyUSID.io.write_utils import Dimension
+from sidpy.hdf.hdf_utils import get_attributes, copy_attributes
+from sidpy.viz import plot_utils
+from pyUSID import Dimension
 import h5py
 
 from matplotlib import pyplot as plt
 
 
-class FFtrEFM(usid.Process):
+class FFtrEFM(Process):
     """
     Implements the pixel-by-pixel processing using ffta.pixel routines
     Abstracted using the Process class for parallel processing
@@ -88,7 +91,7 @@ class FFtrEFM(usid.Process):
         self.parm_dict = parm_dict
 
         if not any(parm_dict):
-            self.parm_dict = usid.hdf_utils.get_attributes(h5_main)
+            self.parm_dict = get_attributes(h5_main)
             self.parm_dict.update({'if_only': if_only})
 
         for key, val in parm_dict.items():
@@ -201,10 +204,9 @@ class FFtrEFM(usid.Process):
 
         ds_shape = [num_rows * num_cols, pnts_per_avg]
 
-        self.h5_results_grp = usid.hdf_utils.create_results_group(self.h5_main, self.process_name)
+        self.h5_results_grp = create_results_group(self.h5_main, self.process_name)
 
-        # h5_meas_group = usid.hdf_utils.create_indexed_group(self.h5_main.parent, self.process_name)
-        usid.hdf_utils.copy_attributes(self.h5_main.parent, self.h5_results_grp)
+        copy_attributes(self.h5_main.parent, self.h5_results_grp)
 
         # Create dimensions
         pos_desc = [Dimension('X', 'm', np.linspace(0, self.parm_dict['FastScanSize'], num_cols)),
@@ -215,7 +217,7 @@ class FFtrEFM(usid.Process):
         # ds_spec_inds, ds_spec_vals = build_ind_val_matrices(spec_desc, is_spectral=True)
 
         # Writes main dataset
-        self.h5_if = usid.hdf_utils.write_main_dataset(self.h5_results_grp,
+        self.h5_if = write_main_dataset(self.h5_results_grp,
                                                        ds_shape,
                                                        'Inst_Freq',  # Name of main dataset
                                                        'Frequency',  # Physical quantity contained in Main dataset
@@ -225,7 +227,7 @@ class FFtrEFM(usid.Process):
                                                        dtype=np.float32,  # data type / precision
                                                        main_dset_attrs=self.parm_dict)
 
-        self.h5_amp = usid.hdf_utils.write_main_dataset(self.h5_results_grp,
+        self.h5_amp = write_main_dataset(self.h5_results_grp,
                                                         ds_shape,
                                                         'Amplitude',  # Name of main dataset
                                                         'Amplitude',  # Physical quantity contained in Main dataset
@@ -240,7 +242,7 @@ class FFtrEFM(usid.Process):
                                                         dtype=np.float32,  # data type / precision
                                                         main_dset_attrs=self.parm_dict)
 
-        self.h5_phase = usid.hdf_utils.write_main_dataset(self.h5_results_grp,
+        self.h5_phase = write_main_dataset(self.h5_results_grp,
                                                           ds_shape,
                                                           'Phase',  # Name of main dataset
                                                           'Phase',  # Physical quantity contained in Main dataset
@@ -255,7 +257,7 @@ class FFtrEFM(usid.Process):
                                                           dtype=np.float32,  # data type / precision
                                                           main_dset_attrs=self.parm_dict)
 
-        self.h5_pwrdis = usid.hdf_utils.write_main_dataset(self.h5_results_grp,
+        self.h5_pwrdis = write_main_dataset(self.h5_results_grp,
                                                            ds_shape,
                                                            'PowerDissipation',  # Name of main dataset
                                                            'Power',  # Physical quantity contained in Main dataset
@@ -278,9 +280,12 @@ class FFtrEFM(usid.Process):
 
         return
 
-    def reshape(self):
+    def reshape(self, cal=None):
+        
         '''
         Reshapes the tFP and shift data to be a matrix, then saves that dataset instead of the 1D
+        
+        cal : UNivariateSpline file from ffta.simulation.cal_curve
         '''
 
         h5_tfp = self.h5_tfp[()]
@@ -295,8 +300,27 @@ class FFtrEFM(usid.Process):
         del self.h5_tfp.file[self.h5_tfp.name]
         del self.h5_tfp.file[self.h5_shift.name]
 
-        self.h5_tfp = self.h5_results_grp.create_dataset('tfp', data=h5_tfp, dtype=np.float32)
-        self.h5_shift = self.h5_results_grp.create_dataset('shift', data=h5_shift, dtype=np.float32)
+        self.h5_tfp = self.h5_results_grp.create_dataset('tfp', 
+                                                         data=h5_tfp, 
+                                                         dtype=np.float32)
+        self.h5_shift = self.h5_results_grp.create_dataset('shift', 
+                                                           data=h5_shift, 
+                                                           dtype=np.float32)
+
+        if cal:
+            
+            if 'tfp_cal' in self.h5_results_grp:
+                del self.h5_results_grp['tfp_cal']
+            tfp_cal = np.exp(cal(np.log(h5_tfp))) # cal spline is by default in log space
+            self.h5_tfp_cal = self.h5_results_grp.create_dataset('tfp_cal', 
+                                                                 data=tfp_cal, 
+                                                                 dtype=np.float32)
+            tfps = np.linspace(self.h5_tfp[()].min(), self.h5_tfp[()].max(), 50)
+            if 'cal_curve' in self.h5_results_grp:
+                del self.h5_results_grp['cal_curve']
+            _ = self.h5_results_grp.create_dataset('cal_curve',
+                                                   data = np.array([tfps, cal(np.log(tfps))]),
+                                                   dtype=np.float32)
 
         return
 
@@ -336,7 +360,7 @@ class FFtrEFM(usid.Process):
 
         if not self.override:
 
-            self.h5_results_grp = usid.hdf_utils.find_dataset(self.h5_main.parent, 'Inst_Freq')[index].parent
+            self.h5_results_grp = find_dataset(self.h5_main.parent, 'Inst_Freq')[index].parent
             self.h5_new_spec_vals = self.h5_results_grp['Spectroscopic_Values']
             self.h5_tfp = self.h5_results_grp['tfp']
             self.h5_shift = self.h5_results_grp['shift']
@@ -345,6 +369,7 @@ class FFtrEFM(usid.Process):
             self.h5_phase = self.h5_results_grp['Phase']
             try:
                 self.h5_pwrdis = self.h5_results_grp['PowerDissipation']
+                self.h5_tfp_cal = self.h5_results_grp['tfp_cal']
             except:
                 pass
         return
@@ -392,7 +417,7 @@ class FFtrEFM(usid.Process):
         return [inst_freq, amplitude, phase, tfp, shift, pwr_diss]
 
 
-def save_CSV_from_file(h5_file, h5_path='/', append='', mirror=False):
+def save_CSV_from_file(h5_file, h5_path='/', append='', mirror=False, offset=0):
     """
     Saves the tfp, shift, and fixed_tfp as CSV files
     
@@ -406,6 +431,9 @@ def save_CSV_from_file(h5_file, h5_path='/', append='', mirror=False):
     
     append : str, optional
         text to append to file name
+        
+    offset : float
+        if calculating tFP with a fixed offset for fitting, this subtracts it out
     """
 
     h5_ff = h5_file
@@ -419,27 +447,45 @@ def save_CSV_from_file(h5_file, h5_path='/', append='', mirror=False):
         print('Saving from pyUSID object')
         h5_ff = h5_file.file
 
-    tfp = usid.hdf_utils.find_dataset(h5_ff[h5_path], 'tfp')[0][()]
-    # tfp_fixed = usid.hdf_utils.find_dataset(h5_file[h5_path], 'tfp_fixed')[0][()]
-    shift = usid.hdf_utils.find_dataset(h5_ff[h5_path], 'shift')[0][()]
-
+    tfp = find_dataset(h5_ff[h5_path], 'tfp')[0][()]
+    shift = find_dataset(h5_ff[h5_path], 'shift')[0][()]
+    
+    try:
+        tfp_cal = find_dataset(h5_ff[h5_path], 'tfp_cal')[0][()]
+    except:
+        tfp_cal = None
+        
     tfp_fixed, _ = badpixels.fix_array(tfp, threshold=2)
     tfp_fixed = np.array(tfp_fixed)
+    
+    if isinstance(tfp_cal, np.ndarray):
+        
+        tfp_cal_fixed, _ = badpixels.fix_array(tfp_cal, threshold=2)
+        tfp_cal_fixed = np.array(tfp_cal_fixed)
 
-    print(usid.hdf_utils.find_dataset(h5_ff[h5_path], 'shift')[0].parent.name)
+    print(find_dataset(h5_ff[h5_path], 'shift')[0].parent.name)
 
     path = h5_ff.file.filename.replace('\\', '/')
     path = '/'.join(path.split('/')[:-1]) + '/'
     os.chdir(path)
 
     if mirror:
-        np.savetxt('tfp-' + append + '.csv', np.fliplr(tfp).T, delimiter=',')
+        np.savetxt('tfp-' + append + '.csv', np.fliplr(tfp - offset).T, delimiter=',')
         np.savetxt('shift-' + append + '.csv', np.fliplr(shift).T, delimiter=',')
-        np.savetxt('tfp_fixed-' + append + '.csv', np.fliplr(tfp_fixed).T, delimiter=',')
+        np.savetxt('tfp_fixed-' + append + '.csv', np.fliplr(tfp_fixed - offset).T, delimiter=',')
     else:
-        np.savetxt('tfp-' + append + '.csv', tfp.T, delimiter=',')
+        np.savetxt('tfp-' + append + '.csv', (tfp- offset).T, delimiter=',')
         np.savetxt('shift-' + append + '.csv', shift.T, delimiter=',')
-        np.savetxt('tfp_fixed-' + append + '.csv', tfp_fixed.T, delimiter=',')
+        np.savetxt('tfp_fixed-' + append + '.csv', (tfp_fixed- offset).T, delimiter=',')
+
+    if isinstance(tfp_cal, np.ndarray):
+        
+        if mirror:
+            np.savetxt('tfp_cal-' + append + '.csv', np.fliplr(tfp_cal- offset).T, delimiter=',')
+            np.savetxt('tfp_cal_fixed-' + append + '.csv', np.fliplr(tfp_cal_fixed- offset).T, delimiter=',')
+        else:
+            np.savetxt('tfp_cal-' + append + '.csv', (tfp_cal- offset).T, delimiter=',')
+            np.savetxt('tfp_cal_fixed-' + append + '.csv', (tfp_cal_fixed- offset).T, delimiter=',')
 
     return
 
@@ -451,7 +497,7 @@ def plot_tfp(ffprocess, scale_tfp=1e6, scale_shift=1, threshold=2, **kwargs):
     
     Parameters
     ----------
-    ffprocess : FFtrEFM class object (inherits Process)
+    ffprocess : FFtrEFM class object (inherits Process) or the parent Group
     
     Returns
     -------
@@ -461,9 +507,43 @@ def plot_tfp(ffprocess, scale_tfp=1e6, scale_shift=1, threshold=2, **kwargs):
 
     tfp_ax = a[0][1]
     shift_ax = a[1][1]
+    tfp_cal_ax = a[1][0]
 
-    img_length = ffprocess.parm_dict['FastScanSize']
-    img_height = ffprocess.parm_dict['SlowScanSize']
+    if isinstance(ffprocess, ffta.hdf_utils.process.FFtrEFM):
+
+        img_length = ffprocess.parm_dict['FastScanSize']
+        img_height = ffprocess.parm_dict['SlowScanSize']
+        
+        num_cols = ffprocess.parm_dict['num_cols']
+        num_rows = ffprocess.parm_dict['num_rows']
+        
+        tfp = ffprocess.h5_tfp[()]
+        shift = ffprocess.h5_shift[()]
+        
+        if 'tfp_cal' in ffprocess.h5_tfp.parent:
+            tfp_cal = ffprocess.h5_tfp.parent['tfp_cal'][()]
+            tfp_cal_fixed, _ = badpixels.fix_array(tfp_cal, threshold=threshold)
+        else:
+            tfp_cal_fixed = tfp_cal = np.ones(tfp.shape)
+    
+    elif isinstance(ffprocess, h5py.Group):
+        
+        attr = get_attributes(ffprocess['Inst_Freq'])
+        img_length = attr['FastScanSize']
+        img_height = attr['SlowScanSize']
+        
+        num_cols = attr['num_cols']
+        num_rows = attr['num_rows']
+        
+        tfp = ffprocess['tfp']
+        shift = ffprocess['shift']
+        
+        if 'tfp_cal' in ffprocess:
+            tfp_cal = ffprocess['tfp_cal'][()]
+            tfp_cal_fixed, _ = badpixels.fix_array(tfp_cal, threshold=threshold)
+        else:
+            tfp_cal_fixed = tfp_cal = np.ones(tfp.shape) 
+            
     kwarg = {'origin': 'lower', 'x_vec': img_length * 1e6,
              'y_vec': img_height * 1e6, 'num_ticks': 5, 'stdevs': 3, 'show_cbar': True}
 
@@ -471,13 +551,11 @@ def plot_tfp(ffprocess, scale_tfp=1e6, scale_shift=1, threshold=2, **kwargs):
         if k not in kwargs:
             kwargs.update({k: v})
 
-    num_cols = ffprocess.parm_dict['num_cols']
-    num_rows = ffprocess.parm_dict['num_rows']
     try:
-        ht = ffprocess.h5_main.file['/height/Raw_Data'][:, 0]
+        ht = ffprocess.h5_main.file['/height_000/Raw_Data'][:, 0]
         ht = np.reshape(ht, [num_cols, num_rows]).transpose()
         ht_ax = a[0][0]
-        ht_image, cbar = usid.viz.plot_utils.plot_map(ht_ax, ht * 1e9, cmap='gray', **kwarg)
+        ht_image, cbar = plot_utils.plot_map(ht_ax, ht * 1e9, cmap='gray', **kwarg)
         cbar.set_label('Height (nm)', rotation=270, labelpad=16)
     except:
         pass
@@ -485,15 +563,19 @@ def plot_tfp(ffprocess, scale_tfp=1e6, scale_shift=1, threshold=2, **kwargs):
     tfp_ax.set_title('tFP Image')
     shift_ax.set_title('Shift Image')
 
-    tfp_fixed, _ = badpixels.fix_array(ffprocess.h5_tfp[()], threshold=threshold)
+    tfp_fixed, _ = badpixels.fix_array(tfp, threshold=threshold)
 
-    tfp_image, cbar_tfp = usid.viz.plot_utils.plot_map(tfp_ax, tfp_fixed * scale_tfp,
-                                                       cmap='inferno', **kwargs)
-    shift_image, cbar_sh = usid.viz.plot_utils.plot_map(shift_ax, ffprocess.h5_shift[()] * scale_shift,
-                                                        cmap='inferno', **kwargs)
 
+    tfp_image, cbar_tfp = plot_utils.plot_map(tfp_ax, tfp_fixed * scale_tfp,
+                                              cmap='inferno', **kwargs)
+    shift_image, cbar_sh = plot_utils.plot_map(shift_ax, shift * scale_shift,
+                                               cmap='inferno', **kwargs)
+    tfp_cal_image, cbar_tfp_cal = plot_utils.plot_map(tfp_cal_ax, tfp_cal_fixed * scale_tfp,
+                                                  cmap='inferno', **kwargs)
+    
     cbar_tfp.set_label('Time (us)', rotation=270, labelpad=16)
     cbar_sh.set_label('Frequency Shift (Hz)', rotation=270, labelpad=16)
+    cbar_tfp_cal.set_label('Time Calib. (us)', rotation=90, labelpad=16)
     text = tfp_ax.text(num_cols / 2, num_rows + 3, '')
 
     return fig, a
