@@ -12,9 +12,9 @@ import numpy as np
 import os
 import h5py
 
-import pycroscopy as px
 import pyUSID as usid
 import sidpy
+from sidpy.hdf.hdf_utils import get_attributes
 from pyUSID import Dimension
 
 from ffta.pixel_utils import load
@@ -97,25 +97,29 @@ def load_wrapper(ibw_file_path='', ff_file_path='', ftype='FF', verbose=False,
 
     if not any(ibw_file_path):
         ibw_file_path = sidpy.io.interface_utils.openfile_dialog(caption='Select IBW Image ',
-                                                  file_types='IBW Files (*.ibw)')
+                                                                 file_types='IBW Files (*.ibw)')
 
     if not any(ff_file_path):
         ff_file_path = sidpy.io.interface_utils.openfile_dialog(caption='Select FF config in folder',
-                                                 file_types='Config File (*.cfg)')
+                                                                file_types='Config File (*.cfg)')
         ff_file_path = '/'.join(ff_file_path.split('/')[:-1])
 
     # Igor image file translation into an H5 file
 
     if ibw_file_path:
-        
+
+        if verbose:
+            print('Creating Topo Directory')
         tran = gl_ibw.GLIBWTranslator()
         h5_path = tran.translate(ibw_file_path, ftype=ftype,
-                             verbose=verbose, subfolder=subfolder)
+                                 verbose=verbose, subfolder=subfolder)
+        if verbose:
+            print('Created file', h5_path)
     else:
         h5_path = 'FF_H5'
-        
+
     # Set up FF data
-    h5_path, data_files, parm_dict = load_folder(folder_path=ff_file_path, 
+    h5_path, data_files, parm_dict = load_folder(folder_path=ff_file_path,
                                                  verbose=verbose,
                                                  file_name=h5_path)
 
@@ -126,7 +130,7 @@ def load_wrapper(ibw_file_path='', ff_file_path='', ftype='FF', verbose=False,
     if loadverbose:
         print('*** Copy-Paste below code for IDE access to h5***')
         print('import h5py')
-        print("h5_file = h5py.File(r'"+h5_path+"')")
+        print("h5_file = h5py.File(r'" + h5_path + "')")
         print("h5_avg = h5_file['FF_Group/FF_Avg']")
 
     return h5_path, parm_dict, h5_ff
@@ -142,7 +146,7 @@ def load_folder(folder_path='', xy_scansize=[0, 0], file_name='FF_H5',
     folder_path : string
         Path to folder you want to process
 
-    xy_sscansize : 2-float array
+    xy_scansize : 2-float array
         Width by Height in meters (e.g. [8e-6, 4e-6]), if not in parameters file
 
     file_name : str
@@ -176,7 +180,7 @@ def load_folder(folder_path='', xy_scansize=[0, 0], file_name='FF_H5',
         folder_path = '/'.join(folder_path.split('/')[:-1])
 
     print(folder_path, 'folder path')
-    filelist = os.listdir(folder_path)
+    filelist = sorted(os.listdir(folder_path))
 
     if textload == False:
 
@@ -217,10 +221,12 @@ def load_folder(folder_path='', xy_scansize=[0, 0], file_name='FF_H5',
         parm_dict['SlowScanSize'] = height
 
     # Check ratio is correct
-    ratio = np.round(parm_dict['FastScanSize'] * 1e6, 1) / np.round(parm_dict['SlowScanSize'] * 1e6, 1)
+    ratio = np.round(parm_dict['FastScanSize'] * 1e6, 4) / np.round(parm_dict['SlowScanSize'] * 1e6, 4)
     if n_pixels / len(data_files) != ratio:
         print(ratio)
-        print(parm_dict['FastScanSize'], parm_dict['SlowScanSize'], n_pixels / len(data_files), len(data_files))
+        print(parm_dict['FastScanSize'], parm_dict['SlowScanSize'],
+              n_pixels / len(data_files),
+              len(data_files))
         raise Exception('X-Y Dimensions do not match filelist. Add manually to config file. Check n-pixels.')
 
     # add associated dimension info
@@ -255,7 +261,7 @@ def load_folder(folder_path='', xy_scansize=[0, 0], file_name='FF_H5',
 
 
 def load_FF(data_files, parm_dict, h5_path, verbose=False, loadverbose=True,
-            average=True, mirror=False):
+            average=True, mirror=True):
     """
     Generates the HDF5 file given path to data_files and parameters dictionary
 
@@ -315,9 +321,13 @@ def load_FF(data_files, parm_dict, h5_path, verbose=False, loadverbose=True,
         def_vec = def_vec[:-1]
         # warnings.warn('Time-per-point calculation error')
 
-    # To do: Fix the labels/atrtibutes on the relevant data sets
-    hdf = h5py.File(h5_path)
-    
+    # To do: Fix the labels/attributes on the relevant data sets
+    try:
+        hdf = h5py.File(h5_path, 'r+')
+    except:
+        print('Creating HDF5 file...')
+        hdf = h5py.File(h5_path, 'w')
+
     try:
         ff_group = hdf.create_group('FF_Group')
     except:
@@ -332,7 +342,7 @@ def load_FF(data_files, parm_dict, h5_path, verbose=False, loadverbose=True,
 
     for p in parm_dict:
         ff_group.attrs[p] = parm_dict[p]
-    ff_group.attrs['pnts_per_line'] = num_cols  
+    ff_group.attrs['pnts_per_line'] = num_cols
 
     h5_ff = usid.hdf_utils.write_main_dataset(ff_group,  # parent HDF5 group
                                               (num_rows * num_cols * pnts_per_pixel, pnts_per_avg),
@@ -406,7 +416,7 @@ def load_pixel_averaged_from_raw(h5_file, verbose=True, loadverbose=True):
 
     """
 
-    hdf = px.io.HDFwriter(h5_file)
+    hdf = h5py.File(h5_file)
     h5_main = usid.hdf_utils.find_dataset(hdf.file, 'FF_Raw')[0]
 
     try:
@@ -414,7 +424,7 @@ def load_pixel_averaged_from_raw(h5_file, verbose=True, loadverbose=True):
     except:
         ff_avg_group = usid.hdf_utils.create_indexed_group(h5_main.parent, 'FF_Avg')
 
-    parm_dict = usid.hdf_utils.get_attributes(h5_main.parent)
+    parm_dict = get_attributes(h5_main.parent)
 
     num_rows = parm_dict['num_rows']
     num_cols = parm_dict['num_cols']
@@ -429,10 +439,8 @@ def load_pixel_averaged_from_raw(h5_file, verbose=True, loadverbose=True):
     # Set up the position vectors for the data
     pos_desc = [Dimension('X', 'm', np.linspace(0, parm_dict['FastScanSize'], num_cols)),
                 Dimension('Y', 'm', np.linspace(0, parm_dict['SlowScanSize'], num_rows))]
-    ds_pos_ind, ds_pos_val = build_ind_val_dsets(pos_desc, is_spectral=False)
 
     spec_desc = [Dimension('Time', 's', np.linspace(0, parm_dict['total_time'], pnts_per_avg))]
-    ds_spec_inds, ds_spec_vals = build_ind_val_dsets(spec_desc, is_spectral=True)
 
     for p in parm_dict:
         ff_avg_group.attrs[p] = parm_dict[p]
@@ -472,166 +480,3 @@ def load_pixel_averaged_from_raw(h5_file, verbose=True, loadverbose=True):
     hdf.flush()
 
     return h5_avg
-
-
-def load_pixel_averaged_FF(data_files, parm_dict, h5_path,
-                           verbose=False, loadverbose=True, mirror=False):
-    """
-    Creates a new group FF_Avg where the raw FF files are averaged together
-
-    This function does not process the Raw data and is more useful when the resulting 
-    Raw data matrix is very large (causing memory errors)
-    
-    This is more useful as pixel-wise averages are more relevant in FF-processing
-
-    This Dataset is (n_pixels*n_rows, n_pnts_per_avg)
-
-    Parameters
-    ----------
-    h5_file : h5py File
-        H5 File to be examined. File typically set as h5_file = hdf.file
-        hdf = px.ioHDF5(h5_path), h5_path = path to disk
-
-    verbose : bool, optional
-        Display outputs of each function or not
-
-    loadverbose : Boolean (optional)
-        Whether to print any simple "loading Line X" statements for feedback
-
-    Returns
-    -------
-    h5_avg : Dataset
-        The new averaged Dataset
-
-    """
-
-    hdf = px.io.HDFwriter(h5_path)
-
-    try:
-        ff_avg_group = hdf.file.create_group('FF_Group')
-    except:
-        ff_avg_group = usid.hdf_utils.create_indexed_group(hdf.file['/'], 'FF_Group')
-
-    try:
-        ff_avg_group = hdf.file[ff_avg_group.name].create_group('FF_Avg')
-    except:
-        ff_avg_group = usid.hdf_utils.create_indexed_group(ff_avg_group, 'FF_Avg')
-
-    num_rows = parm_dict['num_rows']
-    num_cols = parm_dict['num_cols']
-    pnts_per_avg = parm_dict['pnts_per_avg']
-    pnts_per_line = parm_dict['pnts_per_line']
-    pnts_per_pixel = parm_dict['pnts_per_pixel']
-    parm_dict['pnts_per_pixel'] = 1  # only 1 average per pixel now
-    parm_dict['pnts_per_line'] = num_cols  # equivalent now with averaged data
-    n_pix = int(pnts_per_line / pnts_per_pixel)
-    dt = 1 / parm_dict['sampling_rate']
-
-    # Set up the position vectors for the data
-    pos_desc = [Dimension('X', 'm', np.linspace(0, parm_dict['FastScanSize'], num_cols)),
-                Dimension('Y', 'm', np.linspace(0, parm_dict['SlowScanSize'], num_rows))]
-    ds_pos_ind, ds_pos_val = build_ind_val_dsets(pos_desc, is_spectral=False)
-
-    spec_desc = [Dimension('Time', 's', np.linspace(0, parm_dict['total_time'], pnts_per_avg))]
-    ds_spec_inds, ds_spec_vals = build_ind_val_dsets(spec_desc, is_spectral=True)
-
-    for p in parm_dict:
-        ff_avg_group.attrs[p] = parm_dict[p]
-    ff_avg_group.attrs['pnts_per_line'] = num_cols  # to change number of pnts in a line
-    ff_avg_group.attrs['pnts_per_pixel'] = 1  # to change number of pnts in a pixel
-
-    h5_avg = usid.hdf_utils.write_main_dataset(ff_avg_group,  # parent HDF5 group
-                                               (num_rows * num_cols, pnts_per_avg),  # shape of Main dataset
-                                               'FF_Avg',  # Name of main dataset
-                                               'Deflection',  # Physical quantity contained in Main dataset
-                                               'V',  # Units for the physical quantity
-                                               pos_desc,  # Position dimensions
-                                               spec_desc,  # Spectroscopic dimensions
-                                               dtype=np.float32,  # data type / precision
-                                               compression='gzip',
-                                               main_dset_attrs=parm_dict)
-
-    # Generates a line from each data file, averages, then saves the data
-
-    for k, n in zip(data_files, np.arange(0, len(data_files))):
-
-        if loadverbose:
-            fname = k.replace('/', '\\')
-            print('####', fname.split('\\')[-1], '####')
-            fname = str(n).rjust(4, '0')
-
-        line_file = load.signal(k)
-
-        _ll = line.Line(line_file, parm_dict, n_pixels=n_pix, pycroscopy=False)
-        _ll = _ll.pixel_wise_avg().T
-
-        if mirror:
-            h5_avg[n * num_cols:(n + 1) * num_cols, :] = np.flipud(_ll[:, :])
-        else:
-            h5_avg[n * num_cols:(n + 1) * num_cols, :] = _ll[:, :]
-
-    if verbose == True:
-        usid.hdf_utils.print_tree(hdf.file, rel_paths=True)
-        h5_avg = usid.hdf_utils.find_dataset(hdf.file, 'FF_Avg')[0]
-
-        print('H5_avg of size:', h5_avg.shape)
-
-    hdf.flush()
-
-    return h5_avg
-
-
-def createHDF5_file(signal, parm_dict, h5_path='', ds_name='FF_Raw'):
-    """
-    Generates the HDF5 file given path to a specific file and a parameters dictionary
-
-    Parameters
-    ----------
-    h5_path : string
-        Path to desired h5 file.
-
-    signal : str, ndarray
-        Path to the data file to be converted or a workspace array
-
-    parm_dict : dict
-        Scan parameters
-
-    Returns
-    -------
-    h5_path: str
-        The filename path to the H5 file create
-
-    """
-
-    sg = signal
-
-    if 'str' in str(type(signal)):
-        sg = load.signal(signal)
-
-    if not any(h5_path):  # if not passed, auto-generate name
-        fname = signal.replace('/', '\\')
-        h5_path = fname[:-4] + '.h5'
-    else:
-        fname = h5_path
-
-    hdf = px.ioHDF5(h5_path)
-    usid.hdf_utils.print_tree(hdf.file)
-
-    ff_group = px.MicroDataGroup('FF_Group', parent='/')
-    root_group = px.MicroDataGroup('/')
-
-    #    fname = fname.split('\\')[-1][:-4]
-    sg = px.MicroDataset(ds_name, data=sg, dtype=np.float32, parent=ff_group)
-
-    if 'pnts_per_pixel' not in parm_dict.keys():
-        parm_dict['pnts_per_avg'] = signal.shape[1]
-        parm_dict['pnts_per_pixel'] = 1
-        parm_dict['pnts_per_line'] = parm_dict['num_cols']
-
-    ff_group.addChildren([sg])
-    ff_group.attrs = parm_dict
-
-    # Get reference for writing the data
-    h5_refs = hdf.writeData(ff_group, print_log=True)
-
-    hdf.flush()
