@@ -1,21 +1,20 @@
 """pixel.py: Contains pixel class."""
 # pylint: disable=E1101,R0902,C0103
 __author__ = "Rajiv Giridharagopal"
-__copyright__ = "Copyright 2020"
+__copyright__ = "Copyright 2021"
 __maintainer__ = "Rajiv Giridharagopal"
 __email__ = "rgiri@uw.edu"
 __status__ = "Development"
 
 import numpy as np
 from scipy import signal as sps
-from scipy import optimize as spo
-from scipy import interpolate as spi
 from scipy import integrate as spg
 
 from ffta.pixel_utils import noise
 from ffta.pixel_utils import parab
 from ffta.pixel_utils import dwavelet
 from ffta.pixel_utils import tfp_calc
+from ffta.nfmd import NFMD
 
 from matplotlib import pyplot as plt
 import pywt
@@ -23,6 +22,7 @@ import pywt
 import time
 import warnings
 
+
 class Pixel:
     """
     Signal Processing to Extract Time-to-First-Peak.
@@ -35,9 +35,9 @@ class Pixel:
     b) Wavelet Transform
     c) Short-Time Fourier Transform (STFT))
 
-    signal_array : (n_points, n_signals) array_like
+    signal_array: (n_points, n_signals) array_like
         2D real-valued signal array, corresponds to a pixel.
-    params : dict, optional
+    params: dict, optional
         Includes parameters for processing, saved by the experiment Required:
 
         trigger = float (in seconds) (required)
@@ -55,14 +55,14 @@ class Pixel:
         wavelet_parameter = int (default: 5)
         recombination = bool (0: Data are for Charging up, 1: Recombination)
         fit_phase = bool (0: fit to frequency, 1: fit to phase)
-    can_params : dict, optional
+    can_params: dict, optional
         Contains the cantilever parameters (e.g. AMPINVOLS).
         see ffta.pixel_utils.load.cantilever_params
-    fit : bool, optional
+    fit: bool, optional
         Find tFP by just raw minimum (False) or fitting product of 2 exponentials (True)
     pycroscopy : bool, optional
         Pycroscopy requires different orientation, so this corrects for this effect.
-    fit_form : str, optional
+    fit_form: str, optional
         Functional form used when fitting. 
         
         One of 
@@ -71,7 +71,7 @@ class Pixel:
             exp: single expential decay
             ringdown: single exponential decay of amplitude, not frequency, scaled to return Q
     
-    method : str, optional
+    method: str, optional
         Method for generating instantaneous frequency, amplitude, and phase response
         
         One of
@@ -80,9 +80,9 @@ class Pixel:
             stft: short time Fourier transform (sliding FFT)
             fv: Feldman Force-Vib method
     
-    filter_amplitude : bool, optional
+    filter_amplitude: bool, optional
         The Hilbert Transform amplitude can sometimes have drive frequency artifact.
-    filter_frequency : bool, optional
+    filter_frequency: bool, optional
         Filters the instantaneous frequency to remove noise peaks
     recombination: bool, optional
         Whether to invert the frequency (during a recombination or positive frequency shift event)
@@ -92,9 +92,9 @@ class Pixel:
         The point where the event occurs.
     total_time: bool, optional
         The total time of the signal
-    sampling_rate : bool, optional
+    sampling_rate: bool, optional
         The sampling rate. Note that sampling_rate * total_time must equal number of samples
-    roi : bool, opertional
+    roi: bool, opertional
         The length of the window to find a minimum frequency peak
         
     Attributes
@@ -158,14 +158,14 @@ class Pixel:
 
     """
 
-    def __init__(self, 
-                 signal_array, 
-                 params={}, 
+    def __init__(self,
+                 signal_array,
+                 params={},
                  can_params={},
-                 fit=True, 
+                 fit=True,
                  pycroscopy=False,
-                 method='hilbert', 
-                 fit_form='product', 
+                 method='hilbert',
+                 fit_form='product',
                  filter_amplitude=False,
                  filter_frequency=False,
                  recombination=False,
@@ -194,6 +194,16 @@ class Pixel:
         self.fft_params = {}  # for STFT
         self.fft_time_res = 20e-6
 
+        # NFMD 
+        self.nfmd_analysis = False
+        self.num_freqs = 2
+        self.window_size = 40
+        self.optimizer_opts = {'lr': 1e-4}
+        self.max_iters = 100
+        self.target_loss = 1e-4
+        self.update_freq = None
+
+        # Misc Settings
         self.phase_fitting = False
         self.check_drive = True
         self.window = 'blackman'
@@ -227,73 +237,73 @@ class Pixel:
             self.n_signals = 1
             self.signal_array = self.signal_array.flatten()
             self.n_points = self.signal_array.shape[0]
-        
+
         self._n_points_orig = self.signal_array.shape[0]
-        
+
         if pycroscopy:
             self.signal_array = signal_array.T
-        
+
         # The copy of the signal we will manipulate
         self.signal = np.copy(self.signal_array)
-        
+
         # Read parameter attributes from parameters dictionary.
         for key, value in params.items():
             setattr(self, key, value)
 
         for key, value in can_params.items():
             setattr(self, key, float(value))
-            
+
         # Overwrite parameters with explicitly passed parameters
-        for key, val in zip(['trigger', 'total_time', 'sampling_rate', 'roi'], 
+        for key, val in zip(['trigger', 'total_time', 'sampling_rate', 'roi'],
                             [trigger, total_time, sampling_rate, roi]):
             if val:
                 setattr(self, key, val)
 
         # Check for missing required parameters
-        if self.trigger == None: # trigger can be 0
+        if self.trigger == None:  # trigger can be 0
             raise KeyError('Trigger must be supplied')
-            
+
         if not self.total_time:
             if not self.sampling_rate:
                 raise KeyError('total_time or sampling_rate must be supplied')
             else:
                 self.total_time = self.sampling_rate * self.n_points
-        
+
         elif not self.sampling_rate:
             self.sampling_rate = int(self.n_points / self.total_time)
-        
-        elif self.total_time != self.n_points / self.sampling_rate  :
-            print(self.n_points / self.sampling_rate )
+
+        elif self.total_time != self.n_points / self.sampling_rate:
+            print(self.n_points / self.sampling_rate)
             print(self.total_time)
             raise ValueError('total_time and sampling_rate mismatch')
-         
+
         if self.total_time < self.trigger:
-            self.trigger = 0.1 * self.total_time 
-         
+            self.trigger = 0.1 * self.total_time
+
         if not self.roi:
             self.roi = 0.3 * (self.total_time - self.trigger)
             warnings.warn('ROI defaulting to 30% post-trigger')
-        
+
         elif self.roi > self.total_time - self.trigger:
             print(self.roi)
             print(self.total_time - self.trigger)
             warnings.warn('roi must not extend beyond the total_time; setting to maximum')
             self.roi = self.total_time - self.trigger
-        
-        self.tidx = int(self.trigger * self.sampling_rate)    
+
+        self.tidx = int(self.trigger * self.sampling_rate)
         self._tidx_orig = self.tidx
         self.tidx_orig = self.tidx
-        
+
         if not hasattr(self, 'drive_freq'):
             self.average()
             self.set_drive()
-        
+
         # Processing parameters    
         if self.filter_frequency:
             self.bandpass_filter = 0  # turns off FIR
 
         # Initialize attributes that are going to be assigned later.
-        self.signal = None
+        self.signal = np.array([])
         self.phase = None
         self.inst_freq = None
         self.tfp = None
@@ -324,37 +334,34 @@ class Pixel:
         """Removes DC components from each signal using FFT."""
 
         self.signal = np.copy(self.signal_array)
-        
+
         if self.n_signals == 1:
-            
             self.signal = np.reshape(self.signal, (self.n_points, self.n_signals))
 
         for i in range(self.n_signals):
-            
-            f_ax = np.linspace(-self.sampling_rate/2, self.sampling_rate/2, self.n_points)
+            f_ax = np.linspace(-self.sampling_rate / 2, self.sampling_rate / 2, self.n_points)
             mid = int(len(f_ax) / 2)
             # drive_bin = np.searchsorted(f_ax[mid:], self.drive_freq) + mid
             delta_freq = self.sampling_rate / self.n_points
-    
+
             SIG_DC = np.fft.fftshift(np.fft.fft(self.signal[:, i]))
-    
+
             SIG_DC[:mid - int(dc_width / delta_freq)] = 0
             SIG_DC[mid + int(dc_width / delta_freq):] = 0
             sig_dc = np.real(np.fft.ifft(np.fft.ifftshift(SIG_DC)))
-    
+
             self.signal[:, i] -= sig_dc
-        
+
         if plot:
             fig, ax = plt.subplots(nrows=2, figsize=(6, 10))
-            ax[0].plot(np.arange(0, self.total_time, 1/self.sampling_rate), sig_dc, 'b')
-            ax[1].plot(np.arange(0, self.total_time, 1/self.sampling_rate), self.signal_array, 'b')
-            ax[1].plot(np.arange(0, self.total_time, 1/self.sampling_rate), self.signal, 'r')
+            ax[0].plot(np.arange(0, self.total_time, 1 / self.sampling_rate), sig_dc, 'b')
+            ax[1].plot(np.arange(0, self.total_time, 1 / self.sampling_rate), self.signal_array, 'b')
+            ax[1].plot(np.arange(0, self.total_time, 1 / self.sampling_rate), self.signal, 'r')
             plt.title('DC Offset')
-            
+
         if self.n_signals == 1:
-            
-            self.signal = self.signal[:,0]
-            
+            self.signal = self.signal[:, 0]
+
         return
 
     def phase_lock(self):
@@ -373,7 +380,7 @@ class Pixel:
         """Averages signals."""
 
         if self.n_signals != 1:  # if not multi-signal, don't average
-        
+
             # self.signal = self.signal_array.mean(axis=1)
             self.signal = self.signal.mean(axis=1)
 
@@ -388,9 +395,9 @@ class Pixel:
         signal = self.signal[:n_fft]
         fft_amplitude = np.abs(np.fft.rfft(signal))
         drive_freq = fft_amplitude.argmax() * dfreq
-        
+
         self.drive_freq = drive_freq
-        
+
         return
 
     def check_drive_freq(self):
@@ -608,18 +615,18 @@ class Pixel:
                 # Remove the drive from phase.
                 # self.phase -= (2 * np.pi * self.drive_freq *
                 #               np.arange(self.n_points) / self.sampling_rate)
-    
+
                 # A curve fit on the initial part to make sure that it worked.
                 start = int(0.3 * self.tidx)
                 end = int(0.7 * self.tidx)
                 fit = self.phase[start:end]
-    
+
                 xfit = np.polyfit(np.arange(start, end), fit, 1)
-    
+
                 # Remove the fit from phase.
                 self.phase -= (xfit[0] * np.arange(self.n_points)) + xfit[1]
         except:
-            self.phase -= (2 * np.pi * self.drive_freq * 
+            self.phase -= (2 * np.pi * self.drive_freq *
                            np.arange(self.n_points) / self.sampling_rate)
 
         self.phase = -self.phase  # need to correct for negative in DDHO solution
@@ -646,7 +653,7 @@ class Pixel:
         self.inst_freq = self.inst_freq_raw - self.inst_freq_raw[self.tidx]
 
         return
-    
+
     def calculate_cwt(self, f_center=None, verbose=False, optimize=False, fit=False):
         '''
         Calculate instantaneous frequency using continuous wavelet transfer
@@ -668,7 +675,7 @@ class Pixel:
 
         dt = 1 / self.sampling_rate
         sc = pywt.scale2frequency(self.wavelet, self.scales) / dt
-        
+
         if self.verbose:
             print('Wavelet scale from', np.min(sc), 'to', np.max(sc))
 
@@ -727,24 +734,23 @@ class Pixel:
 
         return
 
-    def calculate_stft(self, nfft=200):
+    def calculate_stft(self, nfft=200, calc_phase=False):
         '''
         Sliding FFT approach
         
         Parameters
         ----------
-        time_res : float, optional
+        self.fft_time_res : float, optional
             What timescale to evaluate each FFT over
                   
-        fit : bool, optional
-            Fits a parabola to the frequency peak to get the actual frequency
-            Otherwise defaults to parabolic interpolation (see parab.fit)
-        
         nfft : int
             Length of FFT calculated in the spectrogram. More points gets much slower
-            but the longer the FFT the finer the frequency bin spacing            
+            but the longer the FFT the finer the frequency bin spacing   
+            
+        calc_phase : bool, optional
+            Calculates teh Phase (not usually needed)
         '''
-        
+
         pts_per_ncycle = int(self.fft_time_res * self.sampling_rate)
 
         if nfft < pts_per_ncycle:
@@ -774,8 +780,11 @@ class Pixel:
         inst_freq = np.pad(inst_freq, (_pre, _post))
         amplitude = np.pad(amplitude, (_pre, _post))
 
-        phase = spg.cumtrapz(inst_freq)
-        phase = np.append(phase, phase[-1])
+        if calc_phase:
+            phase = spg.cumtrapz(inst_freq)
+            phase = np.append(phase, phase[-1])
+        else:
+            phase = np.zeros(len(inst_freq))
         tidx = int(self.tidx * len(inst_freq) / self.n_points)
 
         self.amplitude = amplitude
@@ -794,57 +803,108 @@ class Pixel:
         self.phase = phase
 
         return
-    
-    def forcevib(self):
-        """Calculate the resonance frequency directly via FORCE-VIB method.
-        See
-        """
+
+    def calculate_nfmd(self, calc_phase=False, override_window=True, verbose=True):
+        '''
+        Nonstationary Fourier Mode Decomposition Approach
         
-        return
+        calc_phase : bool, optional
+            Calculates the Phase (not usually needed)
+        override_window: bool, optional
+            Automatically adjusts window to be integer number of cycles
+        verbose : bool, optional
+            Console feedback
     
+        '''
+        if not self.signal.any():
+            self.signal = self.signal_array
+            self.average()
+
+        z = self.signal
+
+        if not override_window:
+
+            win_size_cycle = int(self.sampling_rate / self.drive_freq)
+            self.window_size = (self.window_size // win_size_cycle) * win_size_cycle
+
+            if verbose:
+                print('window size automatically adjusted to ', self.window_size)
+
+        nfmd = NFMD(z / np.std(z),
+                    num_freqs=self.num_freqs,
+                    window_size=self.window_size,
+                    optimizer_opts=self.optimizer_opts,
+                    max_iters=self.max_iters,
+                    target_loss=self.target_loss)
+
+        if verbose:
+            freqs, A, losses, indices = nfmd.decompose_signal(self.update_freq)
+        else:
+            freqs, A, losses, indices = nfmd.decompose_signal()
+
+        dt = 1 / self.sampling_rate
+        self.n_freqs = nfmd.correct_frequencies(dt=dt)
+        self.n_amps = nfmd.compute_amps()
+        self.n_mean = nfmd.compute_mean()
+
+        self.inst_freq = self.n_freqs[:, 0]
+        self.amplitude = self.n_amps[:, 0]
+
+        if calc_phase:
+            phase = spg.cumtrapz(self.inst_freq)
+            self.phase = np.append(phase, phase[-1])
+        else:
+            self.phase = self.n_mean
+
+        return
+
     def find_tfp(self):
         """Calculate tfp and shift based self.fit_form and self.fit selection"""
         ridx = int(self.roi * self.sampling_rate)
-        cut = self.inst_freq[self.tidx:(self.tidx + ridx)]
+        cut = np.copy(self.inst_freq[self.tidx:(self.tidx + ridx)])
         cut -= self.inst_freq[self.tidx]
         self.cut = cut
         t = np.arange(cut.shape[0]) / self.sampling_rate
-        
+
         try:
-            
+
             if not self.fit:
-    
+
                 tfp_calc.find_minimum(self, cut)
-    
+
             elif self.fit_form == 'sum':
-    
+
                 tfp_calc.fit_freq_sum(self, cut, t)
-    
+
             elif self.fit_form == 'exp':
-    
+
                 tfp_calc.fit_freq_exp(self, cut, t)
-    
+
             elif self.fit_form == 'ringdown':
-    
+
                 cut = self.amplitude[self.tidx:(self.tidx + ridx)]
                 tfp_calc.fit_ringdown(self, cut, t)
-    
+
             elif self.fit_form == 'product':
-    
+
                 tfp_calc.fit_freq_product(self, cut, t)
-    
+
             elif self.fit_form == 'phase':
-    
+
                 cut = -1 * (self.phase[self.tidx:(self.tidx + ridx)] - self.phase[self.tidx])
                 tfp_calc.fit_phase(self, cut, t)
 
         except:
-            
+
             self.tfp = np.nan
             self.shift = np.nan
             self.best_fit = np.zeros(cut.shape[0])
             print('error with fitting')
 
+        self.cut += self.inst_freq[self.tidx]
+        self.best_fit += self.inst_freq[self.tidx]
+        del cut
+
         return
 
     def restore_signal(self):
@@ -885,7 +945,7 @@ class Pixel:
 
         return
 
-    def plot(self, newplot=True, raw=False):
+    def plot(self, newplot=True, fit=True):
         """ 
         Quick visualization of best_fit and cut.
         
@@ -893,8 +953,8 @@ class Pixel:
         ----------
         newplot : bool, optional
             generates a new plot (True) or plots on existing plot figure (False)
-        
-        raw : bool, optional
+        fit : bool, opttional
+            Overlays fit on the instantaneous frequency image
         """
 
         if newplot:
@@ -911,10 +971,12 @@ class Pixel:
 
         a[0].plot(tx * 1e3, self.inst_freq[cut[0]:cut[1]], 'r-')
 
-        if self.fit_form == 'ringdown':
-            a[1].plot(tx * 1e3, self.best_fit, 'g--')
-        else:
-            a[0].plot(tx * 1e3, self.best_fit, 'g--')
+        if fit:
+            if self.fit_form == 'ringdown':
+                a[1].plot(tx * 1e3, self.best_fit, 'g--')
+            else:
+                a[0].plot(tx * 1e3, self.best_fit, 'g--')
+
         a[1].plot(tx * 1e3, self.amplitude[cut[0]:cut[1]], 'b')
         a[2].plot(tx * 1e3, self.phase[cut[0]:cut[1]] * 180 / np.pi, 'm')
 
@@ -947,7 +1009,8 @@ class Pixel:
             t1 = time.time()
 
         # Remove DC component, first.
-        self.remove_dc()
+        if self.method != 'nfmd':
+            self.remove_dc()
 
         # Phase-lock signals.
         # self.phase_lock()
@@ -959,8 +1022,7 @@ class Pixel:
         # self.remove_dc()
 
         # Check the drive frequency.
-        if self.check_drive:
-            
+        if self.check_drive and self.method != 'nfmd':
             self.check_drive_freq()
 
         # DWT Denoise
@@ -976,11 +1038,13 @@ class Pixel:
             # Calculate instantenous frequency using sliding FFT
             self.calculate_stft(**self.fft_params)
 
-        elif self.method == 'fv':
-            
-            self.forcevib()
+        elif self.method == 'nfmd':
+
+            # Nonstationary Fourier Mode Decomposition
+            self.calculate_nfmd()
 
         elif self.method == 'hilbert':
+
             # Hilbert transform method
 
             # Apply window.
@@ -1033,7 +1097,6 @@ class Pixel:
 
         # If it's a recombination image invert it to find minimum.
         if self.recombination:
-
             self.inst_freq = self.inst_freq * -1
 
         # Find where the minimum is.
@@ -1041,12 +1104,10 @@ class Pixel:
 
         # Restore the length due to FIR filter being causal
         if self.method == 'hilbert':
-            
             self.restore_signal()
 
         # If it's a recombination image invert it to find minimum.
         if self.recombination:
-            
             self.inst_freq = self.inst_freq * -1
             self.best_fit = self.best_fit * -1
             self.cut = self.cut * -1
